
#ifndef _Instantiator_h
#define _Instantiator_h
#pragma interface

#include "JavaTypeSignature.h"
#include <map>
#include <functional>
class CJavaClassFile;
class CJavaFieldSignature;
class CJavaMethodSignature;
class CPJavaConstraint;
class CAuxMethodSignature;
class CAuxMethodMap;
class ostream;

typedef map<unicode_string, CJavaTypeSignature, less<unicode_string> > 
  InstantiationMap;

//
// Class name : CInstantiator
// Description: This class contains the information necessary to map a list
//    of parameters to a list of types, as well as a set of methods to
//    instantiate various signature types.
//
class CInstantiator
{
public:
  CInstantiator(const unicode_string& pName,
		const CJavaTypeSignature& type);
  CInstantiator(const CJavaClassFile& ptype,
		const deque<CJavaTypeSignature>* params = 0);
  CInstantiator(const CInstantiator &other);
  ~CInstantiator();

  void Instantiate(CJavaTypeSignature& fill,
		   const unicode_string &param) const;
  void Instantiate(CJavaTypeSignature& fill,
		   const CJavaTypeSignature& type) const;
  void Instantiate(CJavaFieldSignature& fill,
		   const CJavaFieldSignature& field) const;
  void Instantiate(CJavaMethodSignature& fill,
		   const CJavaMethodSignature& method) const;
  void Instantiate(CPJavaConstraint& fill, 
		   const CPJavaConstraint& constraint) const;
  void Instantiate(CAuxMethodSignature& fill, 
		   const CAuxMethodSignature& auxMethod) const;
  void Instantiate(CAuxMethodMap& fill, 
		   const CAuxMethodMap& map) const;
 
  int Compare(const CJavaTypeSignature& param, 
	      const CJavaTypeSignature& ref) const;
  bool Equals(const CJavaTypeSignature& param,
	      const CJavaTypeSignature& ref) const
    { return Compare(param, ref) == 0; }

  int Compare(const CJavaFieldSignature& param, 
	      const CJavaFieldSignature& ref) const;
  bool Equals(const CJavaFieldSignature& param,
	      const CJavaFieldSignature& ref) const
    { return Compare(param, ref) == 0; }

  int Compare(const CJavaMethodSignature& param, 
	      const CJavaMethodSignature& ref) const;
  bool Equals(const CJavaMethodSignature& param, 
	      const CJavaMethodSignature& ref) const
    { return Compare(param, ref) == 0; }
  bool EqualsIgnoreReturn(const CJavaMethodSignature& param, 
			  const CJavaMethodSignature& ref) const;
		   
  bool Empty() const { return fMap.size() == 0; }

  void Disassemble(ostream &os) const;
private:
  InstantiationMap fMap;
};

#endif
