#pragma implementation
#include "find_leaks.h"
#include "InstantiationList.h"
#include "Compiler.h"
#include <iostream.h>
#include <deque>

//
//  Default constructor
//  Description: creates a new, empty InstantiationList.
//
CInstantiationList::CInstantiationList()
{
}

//
//  Copy constructor
//
CInstantiationList::CInstantiationList(const CInstantiationList& source)
  : fFinished(source.fFinished),
    fNotFinished(source.fNotFinished)
{
}

//
//  Destructor
//
CInstantiationList::~CInstantiationList() 
{
}

//
//  Method name : AddType
//  Description : Assert the fact that we need to instantiate the IType type.
//      If type is _not_ an IType, it is silently ignored.
//  Checks : IType is instantiable. 
//
void CInstantiationList::AddType(const CJavaTypeSignature& type,
				 CSourceFile *file)
{
  assert(fCompilerAlias);
  CJavaTypeSignature t = fCompilerAlias->FixType(type,file);
  if (!t.IsParameterized()) {
    // Ignore
  } else {
    assert(t.IsInstantiable());
    TypeSet::iterator found = fFinished.find(t);
    if (found == fFinished.end()) {
      fNotFinished.insert(t);
    }
    // Add parameterized parameters.
    for (deque<CJavaTypeSignature>::const_iterator i = 
	   t.GetParameters()->begin(); 
	 i != t.GetParameters()->end(); ++i) {
      if ((*i).IsParameterized())
	AddType(*i,file);
    }
  }
}

//
//  Method name : AddTypes
//  Description : Assert the fact that we need to instantiate the ITypes in
//     "types".  Silently ignores non-parameterized types.
//
void CInstantiationList::AddTypes(const TypeSet& types,
				  CSourceFile *file)
{
  TypeSet::const_iterator i = types.begin();
  TypeSet::const_iterator end = types.end();
  for(; i != end; ++i) {
    AddType(*i,file);
  }
}

//
//  Method name : Finished
//  Description : Return true iff there are no ITypes remaining to 
//     instantiate. 
//
bool 
CInstantiationList::Finished() const
{
  return (fNotFinished.size() == 0);
}

//
//  Method name : StartImplementing
//  Description : Return the signature of an unimplemented IType, with
//     the understanding that that IType will be instantiated.
//  Checks : !Finished()
//
const CJavaTypeSignature& 
CInstantiationList::StartImplementing()
{
  assert(!Finished());
  TypeSet::iterator notFinished = fNotFinished.begin();
  TypeSet::iterator finished = (fFinished.insert(*notFinished)).first;
  fNotFinished.erase(notFinished);
  return *finished;
}

//
//  Method name : operator=
//  Description : The assignment operator
//
CInstantiationList &
CInstantiationList::operator=(const CInstantiationList& source)
{
  fFinished = source.fFinished;
  fNotFinished = source.fNotFinished;
  return *this;
}

void
CInstantiationList::SetCompilerAlias(CCompiler *alias)
{ 
  fCompilerAlias = alias;
}
